@@ -197,8 +197,8 @@ extern void initial_image_hdr(char* device, image_head* image_hdr)
 {
 
     fs_open(device);
-    memcpy(image_hdr->magic, IMAGE_MAGIC, IMAGE_MAGIC_SIZE);
-    memcpy(image_hdr->fs, hfsplus_MAGIC, FS_MAGIC_SIZE);
+    strncpy(image_hdr->magic, IMAGE_MAGIC, IMAGE_MAGIC_SIZE);
+    strncpy(image_hdr->fs, hfsplus_MAGIC, FS_MAGIC_SIZE);
     image_hdr->block_size  = (int)reverseInt(sb.blockSize);
     image_hdr->totalblock  = (unsigned long long)reverseInt(sb.totalBlocks);
     image_hdr->device_size = (unsigned long long)(image_hdr->block_size * image_hdr->totalblock);
