@@ -38,6 +38,7 @@
 #define ufs_MAGIC "UFS"
 #define vmfs_MAGIC "VMFS"
 #define jfs_MAGIC "JFS"
+#define btrfs_MAGIC "BTRFS"
 #define raw_MAGIC "raw"
 
 #define IMAGE_VERSION "0001"
@@ -83,6 +84,8 @@ struct cmd_opt
     int force;
     int ignore_fschk;
     int ignore_crc;
+    int quiet;
+    int restore_row_file;
     unsigned long fresh;
 };
 typedef struct cmd_opt cmd_opt;
